@@ -279,13 +279,21 @@ osip_ist_timeout_g_event (osip_transaction_t * ist, osip_event_t * evt)
 {
   osip_via_t *via;
   osip_t *osip = (osip_t *) ist->config;
+#ifndef NEW_TIMER
   time_t now = time (NULL);
+#endif
   int i;
 
   ist->ist_context->timer_g_length = ist->ist_context->timer_g_length * 2;
   if (ist->ist_context->timer_g_length > 4000)
     ist->ist_context->timer_g_length = 4000;
+#ifdef NEW_TIMER
+  gettimeofday(&ist->ist_context->timer_g_start, NULL);
+  add_gettimeofday(&ist->ist_context->timer_g_start,
+		   ist->ist_context->timer_g_length);
+#else
   ist->ist_context->timer_g_start = now;
+#endif
 
   /* retransmit RESPONSE */
   via = (osip_via_t *) osip_list_get (ist->last_response->vias, 0);
@@ -342,7 +350,11 @@ void
 osip_ist_timeout_h_event (osip_transaction_t * ist, osip_event_t * evt)
 {
   ist->ist_context->timer_h_length = -1;
+#ifdef NEW_TIMER
+  ist->ist_context->timer_h_start.tv_sec = -1;
+#else
   ist->ist_context->timer_h_start = -1;
+#endif
 
   __osip_transaction_set_state (ist, IST_TERMINATED);
   __osip_kill_transaction_callback (OSIP_IST_KILL_TRANSACTION, ist);
@@ -352,7 +364,11 @@ void
 osip_ist_timeout_i_event (osip_transaction_t * ist, osip_event_t * evt)
 {
   ist->ist_context->timer_i_length = -1;
+#ifdef NEW_TIMER
+  ist->ist_context->timer_i_start.tv_sec = -1;
+#else
   ist->ist_context->timer_i_start = -1;
+#endif
 
   __osip_transaction_set_state (ist, IST_TERMINATED);
   __osip_kill_transaction_callback (OSIP_IST_KILL_TRANSACTION, ist);
@@ -560,8 +576,17 @@ ist_snd_3456xx (osip_transaction_t * ist, osip_event_t * evt)
 	__osip_message_callback (OSIP_IST_STATUS_6XX_SENT, ist, ist->last_response);
     }
 
+#ifdef NEW_TIMER
+  gettimeofday(&ist->ist_context->timer_g_start, NULL);
+  add_gettimeofday(&ist->ist_context->timer_g_start,
+		   ist->ist_context->timer_g_length);
+  gettimeofday(&ist->ist_context->timer_h_start, NULL);
+  add_gettimeofday(&ist->ist_context->timer_h_start,
+		   ist->ist_context->timer_h_length);
+#else
   ist->ist_context->timer_g_start = time (NULL);
   ist->ist_context->timer_h_start = time (NULL);
+#endif
   __osip_transaction_set_state (ist, IST_COMPLETED);
   return;
 }
@@ -581,6 +606,12 @@ ist_rcv_ack (osip_transaction_t * ist, osip_event_t * evt)
   else				/* IST_CONFIRMED */
     __osip_message_callback (OSIP_IST_ACK_RECEIVED_AGAIN, ist, ist->ack);
   /* set the timer to 0 for reliable, and T4 for unreliable (already set) */
+#ifdef NEW_TIMER
+  gettimeofday(&ist->ist_context->timer_i_start, NULL);
+  add_gettimeofday(&ist->ist_context->timer_i_start,
+		   ist->ist_context->timer_i_length);
+#else
   ist->ist_context->timer_i_start = time (NULL);	/* not started */
+#endif
   __osip_transaction_set_state (ist, IST_CONFIRMED);
 }
