@@ -57,21 +57,29 @@ __osip_nict_init (osip_nict_t ** nict, osip_t * osip, osip_message_t * request)
 
     i = osip_strncasecmp (proto, "TCP", 3);
     if (i != 0)
-      {				/* for other reliable protocol than TCP, the timer
-				   must be desactived by the external application */
+      {
 	(*nict)->timer_e_length = DEFAULT_T1;
-	(*nict)->timer_e_start = now;	/* started */
-
 	(*nict)->timer_k_length = DEFAULT_T4;
+#ifdef NEW_TIMER
+	gettimeofday(&(*nict)->timer_e_start, NULL);
+	add_gettimeofday(&(*nict)->timer_e_start, (*nict)->timer_e_length);
+	(*nict)->timer_k_start.tv_sec = -1; /* not started */
+#else
+	(*nict)->timer_e_start = now;	/* started */
 	(*nict)->timer_k_start = -1;	/* not started */
+#endif
       }
     else
       {				/* TCP is used: */
 	(*nict)->timer_e_length = -1;	/* E is not ACTIVE */
-	(*nict)->timer_e_start = -1;
-
 	(*nict)->timer_k_length = 0;	/* MUST do the transition immediatly */
+#ifdef NEW_TIMER
+	(*nict)->timer_e_start.tv_sec = -1;
+	(*nict)->timer_k_start.tv_sec = -1; /* not started */
+#else
+	(*nict)->timer_e_start = -1;
 	(*nict)->timer_k_start = -1;	/* not started */
+#endif
       }
   }
 
@@ -90,7 +98,12 @@ __osip_nict_init (osip_nict_t ** nict, osip_t * osip, osip_message_t * request)
     (*nict)->port = 5060;
 
   (*nict)->timer_f_length = 64 * DEFAULT_T1;
+#ifdef NEW_TIMER
+  gettimeofday(&(*nict)->timer_f_start, NULL);
+  add_gettimeofday(&(*nict)->timer_f_start, (*nict)->timer_f_length);
+#else
   (*nict)->timer_f_start = now;	/* started */
+#endif
 
   /* Oups! a Bug! */
   /*  (*nict)->port  = 5060; */
@@ -128,6 +141,8 @@ osip_nict_set_destination (osip_nict_t * nict, char *destination, int port)
   return 0;
 }
 
+#ifndef NEW_TIMER
+
 osip_event_t *
 __osip_nict_need_timer_e_event (osip_nict_t * nict, state_t state, int transactionid)
 {
@@ -178,3 +193,61 @@ __osip_nict_need_timer_k_event (osip_nict_t * nict, state_t state, int transacti
     }
   return NULL;
 }
+
+#else
+
+osip_event_t *
+__osip_nict_need_timer_e_event (osip_nict_t * nict, state_t state, int transactionid)
+{
+  struct timeval now;
+  gettimeofday(&now, NULL);
+
+  if (nict == NULL)
+    return NULL;
+  if (state == NICT_PROCEEDING || state == NICT_TRYING)
+    {
+      if (nict->timer_e_start.tv_sec == -1)
+	return NULL;
+      if (timercmp(&now, &nict->timer_e_start, > ))
+	return __osip_event_new (TIMEOUT_E, transactionid);
+    }
+  return NULL;
+}
+
+osip_event_t *
+__osip_nict_need_timer_f_event (osip_nict_t * nict, state_t state, int transactionid)
+{
+  struct timeval now;
+  gettimeofday(&now, NULL);
+
+  if (nict == NULL)
+    return NULL;
+  if (state == NICT_PROCEEDING || state == NICT_TRYING)
+    {
+      if (nict->timer_f_start.tv_sec == -1)
+	return NULL;
+      if (timercmp(&now, &nict->timer_f_start, > ))
+	return __osip_event_new (TIMEOUT_F, transactionid);
+    }
+  return NULL;
+}
+
+osip_event_t *
+__osip_nict_need_timer_k_event (osip_nict_t * nict, state_t state, int transactionid)
+{
+  struct timeval now;
+  gettimeofday(&now, NULL);
+
+  if (nict == NULL)
+    return NULL;
+  if (state == NICT_COMPLETED)
+    {
+      if (nict->timer_k_start.tv_sec == -1)
+	return NULL;
+      if (timercmp(&now, &nict->timer_k_start, > ))
+	return __osip_event_new (TIMEOUT_K, transactionid);
+    }
+  return NULL;
+}
+
+#endif
